float hash13(vec3 x){
    x = fract(x * 443.8975);
    x += dot(x, x.yzx + 19.19);
    return fract((x.x + x.y) * x.z);
}

vec3 hash33(vec3 q){
    uvec3 p = uvec3(ivec3(q));
          p = p * uvec3(374761393U, 1103515245U, 668265263U) + p.zxy + p.yzx;
          p = p.yzx * (p.zxy ^ (p >> 3U));

    return vec3(p ^ (p >> 16U)) * (1 / vec3(0xffffffffU));
}

float bayer2(vec2 a){
    a = floor(a);
    return fract(dot(a, vec2(0.5, a.y * 0.75)));
}

#define bayer4(a)   (bayer2( .5*(a))*.25+bayer2(a))
#define bayer8(a)   (bayer4( .5*(a))*.25+bayer2(a))
#define bayer16(a)  (bayer8( .5*(a))*.25+bayer2(a))
#define bayer32(a)  (bayer16(.5*(a))*.25+bayer2(a))
#define bayer64(a)  (bayer32(.5*(a))*.25+bayer2(a))
#define bayer128(a) (bayer64(.5*(a))*.25+bayer2(a))

float rand(vec2 x){
    return fract(sin(dot(x, vec2(12.9898, 78.233))) * 43758.5453);
}

vec2 rand2(vec2 x){
    return fract(sin(dot(x, vec2(12.9898, 78.233))) * vec2(43758.5453, 28001.8384));
}

vec3 rand3(vec2 x){
    return fract(sin(dot(x, vec2(12.9898, 78.233))) * vec3(43758.5453, 28001.8384, 50849.4141));
}

vec4 blue_noise(vec2 uv){
    ivec2 coord = ivec2(fract(uv / noiseTextureResolution) * noiseTextureResolution);
        
    vec4 noise = texel_fetch(noisetex, coord);
    #ifdef TAA  
        noise = fract(noise + frameCounter / hpi);
    #endif

    return noise;
}

float noise_3D(vec3 position){
    float p = floor(position.z);
    float f = position.z - p;

    float noise0 = texture(noisetex, position.xy / noiseTextureResolution + (p + 0) * 120.7123).x;
    float noise1 = texture(noisetex, position.xy / noiseTextureResolution + (p + 1) * 120.7123).x;

    return mix(noise0, noise1, curve(f));
}

float fbm_2(vec2 position, vec2 direction, const int steps){
    float amplitude = 0.5;
    float frequency = 3.75;

    float noise = amplitude * texture(noisetex, position).x;
    float weight = 0;

    for(int i = 0; i < steps; i++){
        position *= rotation(golden_angle);
        direction *= rotation(golden_angle);

        noise += amplitude * texture(noisetex, position).x;
        position = position * frequency + direction;
        
        amplitude *= 0.5;
        weight += amplitude;
    }

    return noise / weight;
}

float fbm_3(vec3 position, vec3 direction, const int steps){
    float amplitude = 0.5;
    float frequency = 3.75;

    float noise = amplitude * noise_3D(position);
    float weight = 0;

    for(int i = 0; i < steps; i++){
        position.xz *= rotation(golden_angle);
        direction.xz *= rotation(golden_angle);

        noise += amplitude * noise_3D(position);
        position = position * frequency + direction;

        amplitude *= 0.5;
        weight += amplitude;
    }

    return noise / weight;
}