#if defined SHADERS_VERTEX
    flat out float material;

    out vec2 coord;
    out vec2 lightmap;
    out vec3 normal;
    out vec4 color;

    out mat3 position;
    out mat3 tbn;

    #include "/program/shaders0/common/animation.glsl"

    void vertex_position(out vec4 gposition, out mat3 position){
        shadow_settings s = pshadow();

        position[0] = trans_mad(gl_ModelViewMatrix, layout_position.xyz);

        position[1] = trans_mad(shadowModelViewInverse, position[0]);
        position[1] = terrain_curvature(position[1]);
        position[1] = animation(position[1]);

        position[2] = position[1] + cameraPosition;

        gposition = trans_mad(shadowModelView, position[1]).xyzz * diagonal4(gl_ProjectionMatrix) + gl_ProjectionMatrix[3];
        gposition.xyz = shadow_distortion(s, gposition.xyz);
    }   

    void vertex_tbn(out mat3 tbn){
        vec3 n = gl_NormalMatrix * layout_normal.xyz;
             n = mat3(shadowModelViewInverse) * n;

        vec3 t = gl_NormalMatrix * (layout_tangent.xyz / layout_tangent.w);
             t = mat3(shadowModelViewInverse) * t;
             
        vec3 b = cross(t, n);

        tbn = mat3(t, b, n);
    }
    //Strasbourg
    void main(){
        material = layout_material.x;
        coord = layout_texcoord.xy;
        lightmap = layout_lightmap.xy / 254.0;
        normal = gl_NormalMatrix * layout_normal.xyz;
        color = layout_color;

        vertex_position(gl_Position, position);
        vertex_tbn(tbn);
    }
#elif defined SHADERS_FRAGMENT
    flat in float material;

    in vec2 coord;
    in vec2 lightmap;
    in vec3 normal;
    in vec4 color;

    in mat3 position;
    in mat3 tbn;

    #include "/program/shaders0/common/material.glsl"
    #include "/program/shaders0/common/water_normal.glsl"

    float caustics(vec3 position, vec3 normal, float ID){
        float ior = 1.0002926;
              ior = ID == 2 ? 1.33283 : ior; 
              ior = ID == 3 ? 1.309 : ior; 
              ior = ID == 4 ? 1.51714 : ior; 
              ior = ID == 5 ? 1.484 : ior;

        vec3 sample_direction = mat3(shadowModelView) * mat3(gbufferModelViewInverse) * normalize(shadowLightPosition);
        vec3 sample_position = position + refract(sample_direction, normal, 1.0 / ior) * 2.5;

        float old_area = length(dFdy(position) * dFdy(position));
        float new_area = length(dFdy(sample_position) * dFdy(sample_position));
    
        #ifdef WATER_CAUSTICS
            return old_area / new_area * 0.2;
        #else
            return 1.0;
        #endif
    }

    void main(){
        t.textures0 = texture2(tex, coord) * color;

        float ID = 1.0;
        materials(ID, material);

        bool water = ID == 2;

        vec3 shadow_normal;

        if(!water){
            if(t.textures0.w < 0.128) discard;
            shadow_normal = normal;
        } else {
            t.textures0 = vec4(1.0, 1.0, 1.0, 0.0);
            shadow_normal = tbn * water_normal(position[1], position[2], ID == 2.0);
        }

        float caustics = caustics(position[2], shadow_normal, ID);

        buffers0.xyz = t.textures0.xyz;
        buffers0.w = t.textures0.w < 1.0 ? t.textures0.w : 1.0;

        buffers1.x = encodeNormal(normal) * 0.5 + 0.5;
        buffers1.y = lightmap.y * 0.5 + 0.5;
        buffers1.z = caustics * 0.5 + 0.5;
        buffers1.w = ID / 255.0;

        /* RENDERTARGETS:0,1 */
    }
#endif