vec3 sun(atmosphere_constant ac, vec3 position, vec3 direction){
    const vec3 a = vec3(0.397, 0.503, 0.652);
	const vec3 h = a * 0.5;
	const vec3 n = vec3(0.83438, 0.79904, 0.75415);

    float theta = dot(position, direction);
    float disc = saturate(acos(theta) / ac.sun_angular_radius);

    vec3 sun_disc = exp2(log2(1.0 - disc * disc) * h) / n;

    #if ATMOSPHERE_TYPE == 0
        vec3 sunlight = atmosphere_transmittance(ac, position, direction) * rpi * ac.sun_luminance;
    #elif ATMOSPHERE_TYPE == 1
        vec3 sunlight = ac.sun_luminance * rpi;
    #endif

    return theta < cos(ac.sun_angular_radius) ? vec3(0.0) : (sunlight / ac.sun_illuminance) * sun_disc;
}

vec3 moon(atmosphere_constant ac, vec3 position, vec3 sun_position, vec3 moon_position){
    const float roughness = 0.9;            //Between 0.7 and 0.95
    const float ior = 1.565;                //Ior of Feldspar, a type of stone on the surface of the Moon

    vec2 sphere = ray_sphere_intersection(-moon_position, position, ac.moon_angular_radius);
    vec3 normal = normalize(position * sphere.x - moon_position);

    vec3 H = normalize(sun_position - position);

    float NdotL = dot(normal, sun_position);
    float NdotV = dot(normal, -position);
    float LdotH = dot(sun_position, H);
    
    float direct_diffuse = saturate(NdotL) * disney_diffuse(NdotL, NdotV, LdotH, pow2(1.0 - roughness), ior);

    return sphere.y < 0.0 ? vec3(0.0) : vec3(0.7, 0.8, 0.95) * 0.318 * direct_diffuse * ac.sun_luminance;
}

//Stars from nimitz
//https://www.shadertoy.com/view/XtGGRt

vec3 stars(vec3 position, vec3 direction){
    position = position * rotation_mat(vec3(0, 1, 0), direction);

    float sample_resolution = texel.x * 1.7;

    vec3 color = vec3(0);

    for(int i = 0; i < 4; ++i){
        vec3 q = fract(position * (0.15 * sample_resolution.x)) - 0.5;
        vec3 id = floor(position * (0.15 * sample_resolution.x));

        vec2 rn = hash33(id).xy;

        float c2 = 1 - smoothstep(0.0, 0.6, length(q));
              c2 *= fstep(rn.x, 0.0005 + pow2(i) * 0.001);

        color += c2 * (mix(vec3(1.0,0.49,0.1), vec3(0.75,0.9,1.), rn.y) * 0.1 + 0.9);
        position *= 1.3;
    }

    return pow2(color) * 50.0;
}