float ssao(vec3 position, vec3 normal, vec2 noise){
    const int steps = AO_QUALITY;

    const float radius = 0.05 * -position.z;
    const float z_depth = 0.05 * -position.z;

    float occlusion = 1.0;

    for(int i = 0; i < steps; ++i){
        vec2 sample_offset = (i / steps) + noise;
        vec3 direction = normalize(cosine_direction(sample_offset, normal));

        vec3 ray_increment = direction * radius + position;
        vec3 ray_position = view_to_screenspace(ray_increment);

        float depth = texture2(depthtex1, ray_position.xy).x;

        vec3 sample_position = screen_to_viewspace(vec3(ray_position.xy, depth));
        vec3 sample_vector = normalize(sample_position - position);

        float depth0 = linear_depth(ray_increment.z);
        float depth1 = linear_depth(sample_position.z);

        float s_depth = depth1 - depth0; 

        if(s_depth < z_depth && depth0 > 0.0){
            occlusion += 0.75 * saturate(dot(sample_vector, normal));
        }
    }

    occlusion /= steps;
    occlusion = pow(1 - occlusion, 1.5);

    return max0(occlusion);
}

#include "/program/shaders1/common/specular/raytrace.glsl"

float rtao(vec3 view_position, vec3 position, vec3 normal, vec2 noise){
    const int steps0 = 2;
    const int steps1 = 64;

    const float z_depth = 0.1;

    float ao = 1.0;

    for(int i = 0; i < steps0; i++){
        vec2 offset = (i / steps0) + noise;

        vec3 direction = normalize(cosine_direction(offset, normal));
        vec3 view_direction = mat3(gbufferModelView) * direction;

        vec3 hit_position = vec3(0.0);

        float theta = saturate(dot(mat3(gbufferModelView) * normal, view_direction));

        bool intersect = screenspace_raytrace(view_position, position, view_direction, theta, noise.y, z_depth, steps1, hit_position);

        if(intersect){
            ao -= 1.0 / float(steps0);
        } else {
            break;
        }
    }

    ao = pow(ao, 2.0);
    ao = max0(ao);

    return ao;
}