//https://jo.dreggn.org/home/2010_atrous.pdf
//https://github.com/LWJGL/lwjgl3-demos/blob/main/res/org/lwjgl/demo/opengl/raytracing/tutorial8/atrous.fs.glsl

//const bool colortex10MipmapEnabled = true;
//const bool colortex11MipmapEnabled = true;
    
const float n = 24.0;
const float d = 64.0;

const vec2 kernel_3x3[9]  = vec2[9](
    vec2(-1, -1), vec2(0, -1), vec2(1, -1),
    vec2(-1, 0),  vec2(0, 0),  vec2(1, 0),
    vec2(-1, 1),  vec2(0, 1),  vec2(1, 1)
);

vec3 rsm_filter(vec2 coord, const float s){
    const int steps = 9;

    vec3 normal = texture2(colortex11, coord).xyz * 2.0 - 1.0;
    float depth = texture2(colortex10, coord).w;

    vec3 color = vec3(0.0);
    float weight = 0.0;

    for(int i = 0; i < steps; i++){
        vec2 offset = coord + kernel_3x3[i] * inverse_texel * s;

        vec3 s_normal = texture2(colortex11, offset).xyz * 2.0 - 1.0;
        float s_depth = texture2(colortex10, offset).w;

        float w_normal = saturate(dot(normal, s_normal));
              w_normal = pow(w_normal, n);

        float w_depth = abs(depth - s_depth) * d;
              w_depth = exp(-w_depth / depth);

        float s_length = length(kernel_3x3[i]);
        float s_weight = filter_gaussian(s_length, 2.0) * w_normal * w_depth;

        color += texture2(colortex10, offset).xyz * s_weight;
        weight += s_weight;
    }

    color /= weight;
    color = max0(color);

    #ifdef INDIRECT_FILTER
        return color;
    #else   
        return texture2(colortex10, coord).xyz;
    #endif
}

float ao_filter(vec2 coord, const float s){
    const int steps = 9;

    vec3 normal = texture2(colortex11, coord).xyz * 2.0 - 1.0;
    float depth = texture2(colortex10, coord).w;

    float color = 0.0;
    float weight = 0.0;

    for(int i = 0; i < steps; i++){
        vec2 offset = coord + kernel_3x3[i] * inverse_texel * s;

        vec3 s_normal = texture2(colortex11, offset).xyz * 2.0 - 1.0;
        float s_depth = texture2(colortex10, offset).w;

        float w_normal = saturate(dot(normal, s_normal));
              w_normal = pow(w_normal, n);

        float w_depth = abs(depth - s_depth) * d;
              w_depth = exp(-w_depth / depth);

        float s_length = length(kernel_3x3[i]);
        float s_weight = filter_gaussian(s_length, 2.0) * w_normal * w_depth;

        color += texture2(colortex11, offset).w * s_weight;
        weight += s_weight;
    }

    color /= weight;
    color = max0(color);

    return color;
}