vec3 rsm_water_color(water_settings ws, vec2 depth, float color){
    float translucent_depth0 = (depth.x * 8.0 - 4.0) * shadowProjectionInverse[2].z + shadowProjectionInverse[3].z;
    float translucent_depth1 = (depth.y * 8.0 - 4.0) * shadowProjectionInverse[2].z + shadowProjectionInverse[3].z;

    float density = translucent_depth1 - translucent_depth0;
    float caustics_depth = saturate(density * 2.0);

    #ifdef WATER_CAUSTICS
        float caustics = color * 2.0 - 1.0;
              caustics = pow(caustics * 8.0, WATER_CAUSTICS_STRENGTH);
              caustics = caustics + caustics_depth * (1.0 - caustics_depth);
    #else
        float caustics = 1.0;
    #endif

    return exp2(ws.attenuation_coefficient * density);
}

vec3 rsm(materials m, vec3 position, vec3 normal, vec3 sun_direction, vec3 moon_direction, float noise, float lightmap){
    shadow_settings s = pshadow();
    water_settings ws = swater();

    const float noise_size = pow2(8.0);

    float noise0 = noise * noise_size + 0.5;
    float noise1 = noise0 / noise_size;

    vec2 dir = sincos(noise0 * golden_angle);
    mat2 rot = rotation(noise_size * golden_angle);

    vec3 shadow_position = shadow_position(position);
    vec3 shadow_normal = mat3(shadowModelView) * normal * vec3(1.0, 1.0, -1.0);
    vec3 sunlight = transmittance_color(sun_direction, moon_direction);

    vec3 color = vec3(0.0);

    const float radius_squared = 2.0 * s.radius;
    const float radius_offset = radius_squared / 2048.0;
    const float radius_inverse = sqrt(1.0 / radius_squared * pi);

    for(int i = 0; i < s.steps2; i++, dir *= rot){
        vec2 offset = dir * (i + noise1) / s.steps2;
             
        vec2 offset_shadow = shadow_position.xy + offset * radius_offset;
        vec2 offset_dist = shadow_distortion(s, offset_shadow) * 0.5 + 0.5;

        float depth0 = texture2(shadowtex0, offset_dist).x;
        float depth1 = texture2(shadowtex1, offset_dist).x;
          
        vec4 color0 = texture2(shadowcolor0, offset_dist);
        vec4 color1 = texture2(shadowcolor1, offset_dist);

        float s_lightmap = color1.y * 2.0 - 1.0;
        float caustics = color1.z;
        float ID = color1.w * 255;

        bool water = material_ID(ID, 2);

        float alpha = (1.0 - color0.w);

        vec3 albedo = water ? rsm_water_color(ws, vec2(depth0, depth1), color1.z) : srgb_to_linear(color0.xyz);
        #ifdef COLORED_SHADOW
             albedo = mix(albedo, albedo * alpha, saturate(depth1 - depth0));
        #endif
             albedo = pow(length(albedo), 0.8) * normalize(albedo + 0.0001);

        vec3 s_normal = decodeNormal(color1.x * 2.0 - 1.0);
             s_normal.xy = -s_normal.xy;
             
        vec3 s_position = vec3(offset_shadow, depth1 * 8.0 - 4.0) - shadow_position; if(dot(s_position, s_position) > radius_squared) continue;
        vec3 s_vector = f_normalize(s_position);

        float NdotL = saturate(dot(s_vector, shadow_normal));
        float NdotV = saturate(dot(s_vector, s_normal));

        if(NdotL <= 0.0) continue;
        if(NdotV <= 0.0) continue;

        NdotL = m.grass || m.leaves ? 0.5 : NdotL;

        float falloff = 1.0 / (radius_inverse + length(s_position) * 256.0) * 10.0;

        float lightmap_mask = pow2(s_lightmap - lightmap);
              lightmap_mask = saturate(exp2(-lightmap_mask) * 2.2 - 1.2);

        NdotL *= lightmap_mask;
        NdotV *= lightmap_mask;

        color += albedo * NdotL * NdotV * sunlight * falloff;
    }

    #ifdef GI
        return s.brightness * rpi * (color / s.steps2);
    #else
        return vec3(0.0);
    #endif
}