#include "/program/shaders1/common/shadow/pcss.glsl"
#include "/program/shaders1/common/shadow/soft.glsl"

#include "/program/shaders1/common/shadow/contact.glsl"

vec3 shadow(materials m, vec3 position, vec3 normal, float NdotL){
    shadow_settings s = pshadow();
    water_settings ws = swater();

    vec3 shadow = vec3(0.0);
    vec3 shadow_position = shadow_position(position);
    vec3 shadow_normal = mat3(shadowModelView) * clamp(normalize(cross(dFdx(position), dFdy(position))), -1.0, 1.0);

    NdotL = m.grass || m.leaves || m.fire || m.textured ? 0.5 : NdotL;

    float bias = (2048.0 / s.resolution) + tan(acos(NdotL));
          bias = bias * bias_distortion(s, shadow_position.xy) * 0.0007;

    #if SHADOW_TYPE == 0
        shadow_position = shadow_distortion(s, shadow_position) * 0.5 + 0.5;

        vec4 color0 = texture2(shadowcolor0, shadow_position.xy);
        vec4 color1 = texture2(shadowcolor1, shadow_position.xy);

        vec3 s_normal = decodeNormal(color1.x * 2.0 - 1.0);
        float ID = color1.w * 255.0;

        float depth0 = texture2(shadowtex0, shadow_position.xy).x;
        float depth1 = texture2(shadowtex1, shadow_position.xy).x;

        //bias = (depth0 == depth1) ? (dot(shadow_normal, s_normal) > 0.1 ? bias : bias * 0.5) : bias;

        float shadow_depth0 = depth0 > shadow_position.z - bias ? 1.0 : 0.0;
        float shadow_depth1 = depth1 > shadow_position.z - bias ? 1.0 : 0.0;

        color0.xyz = srgb_to_linear(color0.xyz);
        color0.w = (1.0 - color0.w);
            
        float translucent_depth0 = (depth0 * 8.0 - 4.0) * shadowProjectionInverse[2].z + shadowProjectionInverse[3].z;
        float translucent_depth1 = (depth1 * 8.0 - 4.0) * shadowProjectionInverse[2].z + shadowProjectionInverse[3].z;

        float depth = translucent_depth1 - translucent_depth0;
        float caustics_depth = saturate(depth * 2.0);

        #ifdef WATER_CAUSTICS
            float caustics = color1.z * 2.0 - 1.0;
                  caustics = pow(caustics * 8.0, WATER_CAUSTICS_STRENGTH);
                  caustics = caustics + caustics_depth * (1.0 - caustics_depth);
        #else
            float caustics = 1.0;
        #endif

        vec3 water_color = exp2(ws.attenuation_coefficient * 0.17 * depth) * caustics;

        if(ID == 2){
            shadow += mix(vec3(shadow_depth0), water_color, saturate(shadow_depth1 - shadow_depth0));
        } else {
            #ifdef COLORED_SHADOW
                shadow += mix(vec3(shadow_depth0), color0.xyz * color0.w, saturate(shadow_depth1 - shadow_depth0));
            #else
                shadow += vec3(shadow_depth1);
            #endif
        }
    #elif SHADOW_TYPE == 1 || SHADOW_TYPE == 2
        shadow = soft_shadow(s, m, ws, shadow_position, shadow_normal, bias);
    #endif

    return NdotL < 0.001 ? vec3(1.0) : shadow;
}