#if defined DEFERRED1
    float clouds_2D_noise_mask(clouds_2D_settings pc, vec3 position){
        float animation = frameTimeCounter * pc.time_shift;

        vec2 direction = animation * vec2(1.0, -1.0) / pc.scale_mask;
        vec2 scale_position = position.xz / (128.0 * pc.scale_mask);

        float amplitude = 1.0;
        float frequency = 2.5;
        float weight = 0.0;

        float noise = texture2(noisetex, scale_position - direction).x;

        for(int i = 0; i < pc.octaves0; i++){
            scale_position *= rotation(golden_angle);
            direction *= rotation(golden_angle);

            noise += amplitude * texture2(noisetex, scale_position - direction).x;
            scale_position = scale_position * frequency;
            amplitude *= 0.5;
        }

        noise = noise * pc.coverage * 2.0 - 1.0;

        return noise;
    }

    float clouds_2D_noise_detail(clouds_2D_settings pc, vec3 position){
        float animation = frameTimeCounter * pc.time_shift;

        vec3 direction = animation * vec3(1.0, 0.5, -1.0) / pc.scale_detail;
        vec3 scale_position = position / pc.scale_detail;
        
        float amplitude = 1.0;
        float frequency = 3.75;
        float weight = 0.0;

        float noise = noise_3D(scale_position - direction);

        for(int i = 0; i < pc.octaves1; i++){
            scale_position.xz *= rotation(golden_angle);
            direction.xz *= rotation(golden_angle);

            noise += amplitude * noise_3D(scale_position - direction);
            scale_position = scale_position * frequency;
            amplitude *= 0.5;
        }
        
        return noise * 0.5;
    }   

    float clouds_2D_density(clouds_2D_settings pc, vec3 position){
        const float offset = 1.0 / pc.scale_mask;
        const float scale = 1.0 / (24.0 * pc.scale_mask);

        float n0 = clouds_2D_noise_detail(pc, vec3(position.x + offset, position.y, position.z) * scale);
        float n1 = clouds_2D_noise_detail(pc, vec3(position.x, position.y + offset, position.z) * scale);
        float n2 = clouds_2D_noise_detail(pc, vec3(position.x, position.y + offset, position.z) * scale);

        position = vec3(position.x * n0, position.y * n1, position.z * n2);

        float noise0 = clouds_2D_noise_mask(pc, position);
        float noise1 = clouds_2D_noise_detail(pc, position);

        if(noise0 <= 0.0) return 0.0;

        float result = 2.0 * saturate(noise0 - noise1);
              result = pow4(result);

        return result * pc.density;
    }

    float clouds_2D_transmittance(clouds_2D_settings pc, vec3 position, vec3 direction, const int steps){
        float ray_length = 12.0;
        float transmittance = 0.0;

        for(int i = 0; i < steps; i++, position += direction * ray_length){
            transmittance += clouds_2D_density(pc, position) * ray_length;
            ray_length *= 1.45;
        }

        return transmittance;
    }

    vec4 clouds_2D(vec3 position, vec3 direction, vec3 sun_position, vec3 moon_position, out vec3 dist){
        atmosphere_constant ac = atmosphere();
        clouds_2D_settings pc = s_clouds2D();

        #if !defined VOLUMETRIC_CLOUD && !defined CLOUDS_SHADOW
            pc.coverage = mix(pc.coverage, 0.85, wetness);
            pc.thickness = mix(pc.thickness, 600.0, wetness);
        #endif

        vec3 clouds_scattering;
        float transmittance = 1.0;

        vec3 up_position = vec3(0.0, 1.0, 0.0);
        vec3 view_position = vec3(0.0, ac.planet_radius + cameraPosition.y, 0.0);
        
        float VdotL = dot(position, direction);
        float VdotU = dot(position, up_position);
        float LdotU = dot(direction, up_position);
        
        vec2 sphere = ray_sphere_intersection(view_position, position, ac.planet_radius * 0.9999);
        vec2 sphere_min = ray_sphere_intersection(view_position, position, ac.planet_radius + pc.altitude_min);
        vec2 sphere_max = ray_sphere_intersection(view_position, position, ac.planet_radius + pc.altitude_max);

        if(transmittance <= pc.treshold) return vec4(0.0, 0.0, 0.0, 1.0);
        if((eyeAltitude < pc.altitude_min && sphere.y > 0) || (eyeAltitude > pc.altitude_min && position.y > 0)) return vec4(0.0, 0.0, 0.0, 1.0);

        vec3 clouds_position = position * sphere_min.y + cameraPosition;

        dist = clouds_position;

        float density = clouds_2D_density(pc, clouds_position) * pc.thickness;
        if(density <= 0.0) return vec4(0.0, 0.0, 0.0, 1.0);

        float step_transmittance = exp2(-density);
        float step_transmittance_fraction = 1.0 - step_transmittance;

        float direct_transmittance = clouds_2D_transmittance(pc, clouds_position, direction, 12);
        float indirect_transmittance = clouds_2D_transmittance(pc, clouds_position, up_position, 6) * 0.1;

        float powder = 1.0 - 0.97 * exp(-12.0 * density);

        float direct_powder = mix(powder, 1.0, VdotL * 0.5 + 0.5);
        float indirect_powder = mix(powder, 1.0, VdotU * 0.5 + 0.5);

        direct_powder = mix(direct_powder, 2.0, mie_phase(VdotL, ac.mie_g));

        for(int j = 0; j < pc.steps; j++){
            float a = pow(0.6, j);
            float b = pow(0.2, j);
            float c = pow(0.8, j);

            float clouds_transmittance = transmittance * (step_transmittance_fraction * a);

            float step_direct_transmittance = clouds_transmittance * exp(-direct_transmittance * b);
            float step_indirect_transmittance = clouds_transmittance * exp(-indirect_transmittance * b);

            float direct_phase = clouds_phase(VdotL * c, direct_transmittance);
            float indirect_phase = 2.0 * clouds_phase(VdotU * c, indirect_transmittance);

            clouds_scattering.x += step_direct_transmittance * direct_phase * direct_powder;
            clouds_scattering.y += step_indirect_transmittance * indirect_phase * indirect_powder;
        }

        transmittance *= step_transmittance;
        transmittance = saturate((transmittance - pc.treshold) / (1 - pc.treshold));

        vec3 suncolor = transmittance_color(sun_position, moon_position);
        vec3 skycolor = texture2(colortex8, project_sphere(vec3(0.0, 1.0, 0.0)) / ATMOSPHERE_LOD).xyz;

        #if !defined VOLUMETRIC_CLOUD && !defined CLOUDS_SHADOW
            skycolor = mix(skycolor, skycolor + skycolor.g * 2.0, wetness);
        #endif

        vec3 scattering = clouds_scattering.x * suncolor;
             scattering += clouds_scattering.y * skycolor * 4.0;

        return vec4(scattering, transmittance);
    }
#endif