//https://www.diva-portal.org/smash/get/diva2:1223894/FULLTEXT01.pdf

float height_alter(float altitude, float weather_map){
    float stop_altitude = saturate(weather_map + 0.12);

    float attenuation = saturate(remap(altitude, 0.0, 0.07, 0.0, 1.0));
          attenuation *= saturate(remap(altitude, stop_altitude * 0.2, stop_altitude, 1.0, 0.0));

    return attenuation;
}

float density_alter(float altitude, float weather_map){ 
    float attenuation = altitude * saturate(remap(altitude, 0.0, 0.2, 0.0, 1.0));
          attenuation *= weather_map * 2.0;
          attenuation *= saturate(remap(altitude, 0.9, 1.0, 1.0, 0.0));

    return attenuation * pi;
}

float clouds_3D_noise_mask(clouds_3D_settings vc, vec3 position, vec2 direction){
    vec2 scale_position = position.xz / (128.0 * vc.scale_mask);
    
    float amplitude = 1.0;
    float frequency = 3.5;
    float weight = 0.0;

    float noise = texture2(noisetex, scale_position - direction / vc.scale_mask).x;

    for(int i = 0; i < vc.octaves0; i++){
        scale_position *= rotation(golden_angle);
        direction *= rotation(golden_angle);

        noise += amplitude * texture2(noisetex, scale_position - direction / vc.scale_mask).x;
        scale_position = scale_position * frequency;
        amplitude *= 0.5;
        weight += amplitude;
    }

    noise /= weight;

    return noise;
}

float clouds_3D_noise_detail(clouds_3D_settings vc, vec3 position, vec3 direction){
    vec3 scale_position = position / vc.scale_detail;
    
    float amplitude = 1.0;
    float frequency = 3.75;
    float weight = 0.0;

    float noise = noise_3D(scale_position - direction / vc.scale_detail);

    for(int i = 0; i < vc.octaves1; i++){
        scale_position.xz *= rotation(golden_angle);
        direction.xz *= rotation(golden_angle);

        noise += amplitude * noise_3D(scale_position - direction / vc.scale_detail);
        scale_position = scale_position * frequency;
        amplitude *= 0.5;
        weight += amplitude;
    }

    noise /= weight;
    
    return noise * 0.45;
}

float clouds_3D_density(clouds_3D_settings vc, vec3 position){
    float altitude = (position.y - vc.altitude_min) / vc.thickness;
    float animation = frameTimeCounter * vc.time_shift;

    vec2 direction0 = animation * normalize(vec2(1.0, -1.0));
    vec3 direction1 = animation * normalize(vec3(1.0, 0.5, -1.0));

    float weather_map = clouds_3D_noise_mask(vc, position, direction0);
          weather_map = saturate(weather_map * 3.0 - 2.5);

    float attenuation0 = height_alter(altitude, weather_map);
    float attenuation1 = density_alter(altitude, weather_map);

    float noise_mask = clouds_3D_noise_mask(vc, position, direction0);
    float noise_detail = clouds_3D_noise_detail(vc, position, direction1);

    float local_coverage = texture2(noisetex, (direction0 * 50.0 + position.xz) * 0.0000025).x;
          local_coverage = (local_coverage * 3.0 - 0.5) * 0.5 + 0.5;

    #ifdef LOCAL_COVERAGE
        float global_coverage = local_coverage * 1.25 * mix(vc.coverage, vc.weather_coverage, wetness);
    #else 
        float global_coverage = mix(vc.coverage, vc.weather_coverage, wetness);
    #endif

    noise_mask = noise_mask - global_coverage;
    noise_mask += attenuation0;

    if(noise_mask <= 0.0) return 0.0;

    float result = 2.0 * (noise_mask - noise_detail);
          result = result * attenuation1;

    return saturate(result) * vc.density;
}

float clouds_3D_shadow(vec3 position){
    atmosphere_constant ac = atmosphere();
    clouds_3D_settings vc = s_clouds3D();

    vec3 shadow_position = mat3(shadowModelView) * position;
         shadow_position /= 200.0;
         shadow_position = shadow_position * 0.5 + 0.5;

    return texture2(colortex8, shadow_position.xy / 1024.0).w;
}

#if defined DEFERRED1
    float clouds_3D_transmittance(clouds_3D_settings vc, vec3 position, vec3 direction, const int steps){
        float ray_length = 22.0;
        float transmittance = 0.0;

        for(int i = 0; i < steps; i++, position += direction * ray_length){
            transmittance += clouds_3D_density(vc, position) * ray_length;
            ray_length *= 1.45;
        }

        return transmittance;
    }
//Elsass frei !!
    vec4 clouds_3D(vec3 position, vec3 direction, vec3 sun_position, vec3 moon_position, float noise, const int steps, out vec3 dist){
        atmosphere_constant ac = atmosphere();
        clouds_3D_settings vc = s_clouds3D();

        vec3 clouds_scattering;
        float transmittance = 1.0;

        vec3 up_position = vec3(0.0, 1.0, 0.0);
        vec3 view_position = vec3(0.0, ac.planet_radius + cameraPosition.y, 0.0);
        
        float VdotL = dot(position, direction);
        float VdotU = dot(position, up_position);
        float LdotU = dot(direction, up_position);
        
        vec2 sphere = ray_sphere_intersection(view_position, position, ac.planet_radius * 0.99995);
        vec2 sphere_min = ray_sphere_intersection(view_position, position, ac.planet_radius + vc.altitude_min);
        vec2 sphere_max = ray_sphere_intersection(view_position, position, ac.planet_radius + vc.altitude_max);

        if((eyeAltitude < vc.altitude_min && sphere.y > 0) || (eyeAltitude > vc.altitude_max && position.y > 0)) return vec4(0.0, 0.0, 0.0, 1.0);

        float dist0 = eyeAltitude > vc.altitude_max ? sphere_max.x : sphere_min.y;
        float dist1 = eyeAltitude > vc.altitude_max ? sphere_min.x : sphere_max.y;

        float range = (1 - saturate((eyeAltitude - vc.altitude_max) * 0.1)) * (1 - saturate((vc.altitude_min - eyeAltitude) * 0.1));

        vec3 position0 = position * dist0;
             position0 = position0 * (1.0 - range);
             position0 = clouds_curvature(position0);

        vec3 position1 = position * dist1;
             position1 = mix(position1, position * vc.altitude_min * 10.0, range);
             position1 = clouds_curvature(position1);

        dist = position0;

        vec3 clouds_increment = (position1 - position0) / steps;
        vec3 clouds_position = clouds_increment * noise + position0 + cameraPosition;

        float clouds_length = length(clouds_increment);

        for(int i = 0; i < steps; i++, clouds_position += clouds_increment){
            if(transmittance <= vc.treshold) break;

            float density = clouds_3D_density(vc, clouds_position) * clouds_length;
            if(density <= 0.0) continue;

            float step_transmittance = exp2(-density);
            float step_transmittance_fraction = 1.0 - step_transmittance;

            float direct_transmittance = clouds_3D_transmittance(vc, clouds_position, direction, 12);
            float indirect_transmittance = clouds_3D_transmittance(vc, clouds_position, up_position, 6) * 0.1;
            float bounce_transmittance = clouds_3D_transmittance(vc, clouds_position, -up_position, 4);

            float powder = 1.0 - exp(-2.0 * density);

            float direct_powder = mix(powder, 1.0, VdotL * 0.5 + 0.5);
            float indirect_powder = mix(powder, 1.0, VdotU * 0.5 + 0.5);
            float bounce_powder = mix(powder, 1.0, LdotU * 0.5 + 0.5);

            direct_powder = mix(direct_powder, 2.0, mie_phase(VdotL, ac.mie_g));

            for(int j = 0; j < vc.steps1; j++){
                float a = pow(0.6, j);
                float b = pow(0.4, j);
                float c = pow(0.8, j);

                float clouds_transmittance = (-transmittance * step_transmittance * 0.75 + transmittance) * (step_transmittance_fraction * a);

                float step_direct_transmittance = clouds_transmittance * exp(-direct_transmittance * b);
                float step_indirect_transmittance = clouds_transmittance * exp(-indirect_transmittance * b);
                float step_bounce_transmittance = clouds_transmittance * exp(-bounce_transmittance * b);

                float direct_phase = clouds_phase(VdotL * c, direct_transmittance);
                float indirect_phase = 2.0 * clouds_phase(VdotU * c, indirect_transmittance);

                clouds_scattering.x += step_direct_transmittance * direct_powder * direct_phase;
                clouds_scattering.y += step_indirect_transmittance * indirect_powder * indirect_phase;
                clouds_scattering.z += step_bounce_transmittance * bounce_powder;
            }

            transmittance *= step_transmittance;
        }

        transmittance = saturate((transmittance - vc.treshold) / (1.0 - vc.treshold));

        vec3 suncolor = transmittance_color(sun_position, moon_position);

        vec3 skycolor = texture2(colortex8, project_sphere(vec3(0.0, 1.0, 0.0)) / ATMOSPHERE_LOD).xyz;
             skycolor = mix(skycolor, skycolor + skycolor.g * 1.4, wetness);

        vec3 albedo = vec3(1.0, 1.0, 1.0);

        vec3 scattering = albedo * clouds_scattering.x * suncolor;
             scattering += albedo * clouds_scattering.y * skycolor * 4.0;
             scattering += albedo * clouds_scattering.z * suncolor * abs(LdotU) * 0.02 * rpi * (1.0 - wetness);

        return vec4(scattering, transmittance);
    }
#endif