#if defined SHADERS_VERTEX
    out vec2 coord;

    void main(){
        coord = layout_texcoord.xy;

        gl_Position = vec4(coord * 2.0 - 1.0, 0.0, 1.0);
    }
#elif defined SHADERS_FRAGMENT
    in vec2 coord;

    #include "/program/shaders1/common/atmosphere/atmosphere.glsl"

    #include "/program/shaders1/common/indirect/ao.glsl"
    #include "/program/shaders1/common/indirect/rsm.glsl"

    void main(){
        const float lod = INDIRECT_LOD;

        vec2 scale_coord = coord * lod;
        vec4 noise = blue_noise(gl_FragCoord.xy);

        if(saturate(scale_coord) == scale_coord){
            gbuffers_data g = gbuffers(scale_coord);
            space_position sp = position(scale_coord, g.depth, false);
            materials m = material(g.depth, g.ID);

            vec2 position = screen_velocity(sp.view_position1);
            vec2 velocity = (scale_coord - position);
            vec2 projection = abs(2.0 * fract(velocity * texel) - 0.5) * 2.0;

            vec3 current = rsm(m, sp.inverse_position1, g.normal, sp.sun_vector, sp.moon_vector, noise.w, g.lightmap.y);
                 
            #if AO == 0
                float ao_current = ssao(sp.position1, g.view_normal, noise.xy);
            #elif AO == 1
                float ao_current = rtao(sp.view_position1, sp.position1, g.normal, noise.xy);
            #elif AO == 2
                float ao_current = ssao(sp.position1, g.view_normal, noise.xy) * rtao(sp.view_position1, sp.position1, g.normal, noise.xy);
            #elif AO == 3
                float ao_current = 1.0;
            #endif
                 
            vec3 previous = texture2(colortex10, velocity / lod).xyz;
            float ao_previous = texture2(colortex11, velocity / lod).w;

            float depth0 = linear_depth(g.depth.y);
            float depth1 = texture2(colortex10, velocity / lod).w;

            float weight = saturate(velocity) != velocity ? 0.0 : 0.95;
                  weight *= (projection.x + projection.y) * 0.5;
                  weight *= exp2(-abs(depth0 - depth1) / depth0 * 12.0);
                  weight = saturate(weight);

            buffers0.xyz = mix(current, previous, weight);
            buffers0.w = depth0;

            buffers1.xyz = g.normal * 0.5 + 0.5;
            buffers1.w = mix(ao_current, ao_previous, weight);
        }

        /* RENDERTARGETS:10,11 */
    }
#endif