#if defined SHADERS_VERTEX
    out vec2 coord;

    out mat3 skylight0;
    out mat3 skylight1;

    void main(){
        coord = layout_texcoord.xy;

        gl_Position = vec4(coord * 2.0 - 1.0, 0.0, 1.0);

    }
#elif defined SHADERS_FRAGMENT
    in vec2 coord;

    #include "/program/shaders1/common/atmosphere/atmosphere.glsl"
    #include "/program/shaders1/common/volumetric/clouds_3D.glsl"

    #include "/program/shaders1/common/volumetric/volumetric_shadow.glsl"
    #include "/program/shaders1/common/volumetric/water_fog.glsl"
    #include "/program/shaders1/common/volumetric/volumetric_fog.glsl"

    void main(){
        water_settings ws = swater();

        const float lod = VOLUMETRIC_LOD;

        vec2 scale_coord = coord * lod;
        vec4 noise = blue_noise(gl_FragCoord.xy);

        if(saturate(scale_coord) == scale_coord){
            gbuffers_data g = gbuffers(scale_coord);
            space_position sp = position(scale_coord, g.depth, false);
            materials m = material(g.depth, g.ID);

            g.lightmap.y = saturate(g.lightmap.y);
            g.lightmap.y = 1.0 - pow(1.0 - g.lightmap.y, 0.75);
            g.lightmap.y = pow2(g.lightmap.y);

            vec3 water_previous = texture2(colortex12, coord / lod).xyz;

            vec3 water_transmittance = vec3(1.0);
            vec3 water_scattering = volumetric_water_fog(gbufferModelViewInverse[3].xyz, sp.inverse_position0, sp.light_vector, sp.sun_vector, sp.moon_vector, noise.x, g.lightmap.y, ws.steps, water_transmittance);

            buffers0.xyz = water_scattering;
            buffers0.w = 1.0;

            buffers1.xyz = water_transmittance;
            buffers1.w = 1.0;

            #ifdef VOLUMETRIC_FOG
                #if defined OVER_WORLD
                    buffers2 = volumetric_fog(gbufferModelViewInverse[3].xyz, sp.inverse_position0, sp.light_vector, sp.sun_vector, sp.moon_vector, noise);
                #else
                    buffers2 = vec4(0.0, 0.0, 0.0, 1.0);
                #endif
            #else
                buffers2 = vec4(0.0, 0.0, 0.0, 1.0);
            #endif
        }

        /* RENDERTARGETS:12,13,14 */
    }
#endif