#if defined SHADERS_VERTEX
    out vec2 coord;

    out mat3 skylight0;
    out mat3 skylight1;

    #include "/program/shaders1/common/indirect/skylight.glsl"

    void main(){
        coord = layout_texcoord.xy;

        gl_Position = vec4(coord * 2.0 - 1.0, 0.0, 1.0);

        vertex_skylight();
    }
#elif defined SHADERS_FRAGMENT
    in vec2 coord;

    in mat3 skylight0;
    in mat3 skylight1;

    #include "/program/shaders1/common/atmosphere/atmosphere.glsl"
    #include "/program/shaders1/common/atmosphere/planet.glsl"
    #include "/program/shaders1/common/specular/reflection.glsl"
    #include "/program/shaders1/common/diffuse_light.glsl"

    #include "/program/shaders1/common/volumetric/volumetric_shadow.glsl"
    #include "/program/shaders1/common/volumetric/water_fog.glsl"

    vec3 translucent_render(gbuffers_data g, space_position sp, materials m, vec2 coord, vec3 color){
        vec4 albedo = texture2(colortex1, coord);
             albedo.xyz = srgb_to_linear(albedo.xyz) * albedo.w;
             albedo.w = srgb_to_linear(albedo.w);

        vec4 noise = blue_noise(gl_FragCoord.xy);
        
        if(m.translucent || m.textured){
            if(!m.water){
                color *= albedo.xyz * (1.0 - albedo.w);    
            }

            color += diffuse_light(g, sp, m, coord, albedo.xyz, g.normal, sp.view_position0, sp.position0, sp.inverse_position0, noise);
        }

        return color;
    }

    vec3 atmosphere(vec2 coord, vec3 position, vec3 sun_position, vec3 moon_position){
        atmosphere_constant ac = atmosphere();

        const float lod = ATMOSPHERE_LOD;

        #if defined VOLUMETRIC_CLOUD && defined CLOUDS_SHADOW
            float rain_brightness = 1.0;
        #else
            float rain_brightness = pow5(1.0 - wetness);
        #endif

        float clouds_transmittance = texture2(colortex6, coord / lod).w;

        vec3 scattering = texture2(colortex6, coord / lod).xyz;
        vec3 transmittance = texture2(colortex7, project_sphere(position) / lod).xyz * pow4(clouds_transmittance);
        
        #if ATMOSPHERE_TYPE == 0            
            vec3 color = scattering;

            #if defined OVER_WORLD
                 color += sun(ac, position, sun_position) * transmittance * rain_brightness;
                 color += moon(ac, position, sun_position, moon_position) * luminance(transmittance) * rain_brightness;
                 color += stars(position, moon_position) * luminance(transmittance) * rain_brightness;

                #ifdef SIXSEVEN_PLANET
                    color += planet_sphere(position) * transmittance * rain_brightness;
                #endif
            #elif END_WORLD
                color += sun(ac, position, sun_position) * transmittance;
                color += stars(position, moon_position) * luminance(transmittance);
            #endif

            return color;
        #elif ATMOSPHERE_TYPE == 1
            vec3 color = scattering;
                 color += sun(ac, position, sun_position) * transmittance;
                 color += stars(position, moon_position) * luminance(transmittance);

            #ifdef SIXSEVEN_PLANET
                color += planet_sphere(position) * transmittance;
            #endif

            return color;
        #endif
    }

    vec3 water_fog(gbuffers_data g, space_position sp, materials m, vec3 color){
        g.lightmap.y = saturate(g.lightmap.y);
        g.lightmap.y = 1.0 - pow(1.0 - g.lightmap.y, 0.75);
        g.lightmap.y = pow2(g.lightmap.y);

        if(m.water && isEyeInWater == 0){
            vec4 noise = blue_noise(gl_FragCoord.xy);

            vec3 water_transmittance = vec3(1.0);
            vec3 water_scattering = volumetric_water_fog(sp.inverse_position0, sp.inverse_position1, sp.light_vector, sp.sun_vector, sp.moon_vector, noise.x, g.lightmap.y, 4, water_transmittance);
            
            #ifdef WATER_FOG
                return color * water_transmittance + water_scattering;
            #else
                return color * water_scattering;
            #endif
        } else {
            return color;
        }
    }

    void main(){
        gbuffers_data g = gbuffers(coord);
        space_position sp = position(coord, g.depth, false);
        materials m = material(g.depth, g.ID);
        
        vec3 color = m.atmosphere ? atmosphere(coord, sp.inverse_vector1, sp.sun_vector, sp.moon_vector) : decodeRGBE8(texture2(colortex0, coord));
             color = water_fog(g, sp, m, color);
             color = translucent_render(g, sp, m, coord, color);

        buffers0 = encodeRGBE8(color);

        /* RENDERTARGETS:0 */
    }
#endif