#if defined SHADERS_VERTEX
    out vec2 coord;

    out mat3 skylight0;
    out mat3 skylight1;

    #include "/program/shaders1/common/indirect/skylight.glsl"

    void main(){
        coord = layout_texcoord.xy;

        gl_Position = vec4(coord * 2.0 - 1.0, 0.0, 1.0);

        vertex_skylight();
    }
#elif defined SHADERS_FRAGMENT
    in vec2 coord;

    in mat3 skylight0;
    in mat3 skylight1;

    #include "/program/shaders1/common/atmosphere/atmosphere.glsl"
    #include "/program/shaders1/common/atmosphere/planet.glsl"
    #include "/program/shaders1/common/specular/reflection.glsl"
    #include "/program/shaders1/common/diffuse_light.glsl"

    vec3 translucent_render(gbuffers_data g, space_position sp, materials m, vec2 coord, vec3 color){
        vec4 albedo = texture2(colortex1, coord);
             albedo.xyz = srgb_to_linear(albedo.xyz);
             albedo.w = srgb_to_linear(albedo.w);

        vec4 noise = blue_noise(gl_FragCoord.xy);
        
        if(m.translucent || m.textured){
            if(!m.water) color *= albedo.xyz;

            color *= 1.0 - albedo.w;
            color += diffuse_light(g, sp, m, coord, albedo.xyz, g.normal, sp.view_position0, sp.position0, sp.inverse_position0, noise);
        }

        return color;
    }

    vec3 atmospheric_fog(vec3 color, vec3 position0, vec3 position1, vec3 direction, vec3 sun_position, vec3 moon_position, bool atmosphere_mask){
        atmosphere_constant ac = atmosphere();

        #if ATMOSPHERE_TYPE == 1
            return color;
        #endif

        vec3 scattering = vec3(0.0);
        vec3 transmittance = vec3(1.0);

        float depth = (atmosphere_mask ? far : pow2(distance(position0, position1))) * mix(0.02, 0.3, wetness);

        vec3 density = (ac.attenuation_coefficient[0] + ac.attenuation_coefficient[1] + ac.attenuation_coefficient[2]) * depth;
        vec3 step_transmittance = exp2(-density);
        vec3 step_transmittance_fraction = saturate((step_transmittance - 1.0) / -density);
        vec3 s_transmittance = transmittance * step_transmittance_fraction;

        float theta = dot(f_normalize(position1), direction);
        vec2 phase = vec2(rayleigh_phase(theta), mie_phase(theta, ac.mie_g));

        vec3 sunlight = transmittance_color(sun_position, moon_position);
        vec3 skylight = texture2(colortex8, project_sphere(vec3(0.0, 1.0, 0.0)) / 2.0).xyz * pi;

        float ambient_occlusion = eyeBrightnessSmooth.y / 254.0;
		      ambient_occlusion = pow2(ambient_occlusion);

        scattering += ac.scattering_coefficient * vec2(depth * phase) * s_transmittance * sunlight;
        scattering += ac.scattering_coefficient * vec2(depth) * s_transmittance * (skylight + skylight.g * pi) * ambient_occlusion;
        scattering = mix(scattering, vec3(luminance(scattering)), wetness);

        transmittance *= step_transmittance;

        if(isEyeInWater == 0){
            #if defined OVER_WORLD
                return color * transmittance + scattering;
            #else
                return color;
            #endif
        } else {
            return color;
        }
    }

    vec3 nether_fog(vec3 color, vec3 position0, vec3 position1){
        float depth = distance(position0, position1) * 0.1;

        vec3 transmittance = exp2(-srgb_to_linear(fogColor) * depth);
        vec3 scattering = (1.0 - transmittance) * 12.0;

        return color * transmittance + scattering;
    }

    vec3 water_fog(vec2 coord, vec3 color, bool water){
        if(isEyeInWater == 1){
            vec3 water_scattering = texture2(colortex12, coord / VOLUMETRIC_LOD).xyz;
            vec3 water_transmittance = texture2(colortex13, coord / VOLUMETRIC_LOD).xyz;

            #ifdef WATER_FOG
                return color * water_transmittance + water_scattering;
            #else
                return color * water_scattering;
            #endif
        } else {
            return color;
        }
    }

    vec3 refraction(space_position sp, materials m, vec2 depth, vec3 normal, float ior, bool translucen){
        vec3 normal_flat = clamp(cross(dFdx(sp.position0), dFdy(sp.position0)), -1.0, 1.0);
             normal_flat = normalize(normal_flat);

        vec3 direction = refract(sp.vector0, normal - normal_flat, 1.0 / ior) * 4.0;

        float increment = distance(sp.position1, sp.position0);
              increment = saturate(increment);

        vec3 position = direction * increment + sp.position0;
             position = view_to_screenspace(position);

        float depth0 = texture2(depthtex0, position.xy).x;
        float depth1 = texture2(depthtex1, position.xy).x;

        if(depth1 > depth0 && !m.textured){
            return decodeRGBE8(texture2(colortex0, position.xy));
            //return vec3(0.18, 0.40, 0.58) * 0.25;
        } else {
            return decodeRGBE8(texture2(colortex0, coord));
        }
    }

    void main(){
        gbuffers_data g = gbuffers(coord);
        space_position sp = position(coord, g.depth, false);
        materials m = material(g.depth, g.ID);

        vec4 vl = texture2(colortex14, coord / VOLUMETRIC_LOD);

        vec3 color = refraction(sp, m, g.depth, g.view_normal, m.ior, m.translucent);
             //color = translucent_render(g, sp, m, coord, color);
             color = reflection(m, color, sp.view_position0, sp.position0, sp.inverse_vector0, g.normal, g.albedo, g.depth.x, g.lightmap.y, g.roughness, g.f0);
             color = atmospheric_fog(color, gbufferModelViewInverse[3].xyz, sp.inverse_position0, sp.light_vector, sp.sun_vector, sp.moon_vector, m.atmosphere);
             color = water_fog(coord, color, m.water);

        if(isEyeInWater == 0){
             color = color * vl.w + vl.xyz;
        }

        #if defined NETHER_WORLD
            color = nether_fog(color, gbufferModelViewInverse[3].xyz, sp.inverse_position0);
        #endif

        buffers0.xyz = linear_to_srgb(max0(color));
        buffers0.w = 1.0;

        /* RENDERTARGETS:4 */
    }
#endif