const bool colortex4MipmapEnabled = true;

#if defined SHADERS_VERTEX
    out float exposure;
    out vec2 coord;

    void average_exposure(out float exposure){
        atmosphere_constant ac = atmosphere();
        exposure_settings e = pexposure();

        float lod = log2(max2(texel));

        #if defined OVER_WORLD
            float exposure_min = mix(2.0, 0.1, nightVision);
            float exposure_max = 1200.0;
        #elif defined NETHER_WORLD
            float exposure_min = mix(2.1, 0.1, nightVision);
            float exposure_max = 12.0;
        #else
            float exposure_min = 512.0;
            float exposure_max = 4200.0;
        #endif

        float current = luminance(srgb_to_linear(texture2_lod(colortex4, vec2(0.5), lod).xyz));
              current = clamp(current, exposure_min, exposure_max);
              current = log2(current * 100.0 / e.calibration);
              current = 1.2 * exp2(current - e.offset);

        float previous = texel_fetch(colortex5, ivec2(0.5)).w;

        float exposure_time = current > previous ? e.speed_min : e.speed_max;
        float exposure_frame_time = exp(-exposure_time * frameTime);

        #if EXPOSURE_TYPE == 0
            exposure = mix(current, previous, exposure_frame_time);
        #elif EXPOSURE_TYPE == 1
            exposure = log2(sqrt(e.aperture * 100.0) / (e.shutter_speed * e.iso));
            exposure = 1.2 * exp2(exposure - e.offset);
        #endif
    }

    void main(){
        coord = layout_texcoord.xy;

        gl_Position = vec4(coord * 2.0 - 1.0, 0.0, 1.0);

        average_exposure(exposure);
    }
#elif defined SHADERS_FRAGMENT
    in float exposure;
    in vec2 coord;

    void main(){
        #if EXPOSURE_TYPE == 0
            vec3 color = srgb_to_linear(texture2(colortex4, coord).xyz) / exposure;
        #elif EXPOSURE_TYPE == 1
            vec3 color = srgb_to_linear(texture2(colortex4, coord).xyz) * exposure;
        #endif

        buffers0.xyz = color;
        buffers0.w = 1.0;

        buffers1.xyz = texture2(colortex5, coord).xyz;
        buffers1.w = exposure;

        /* RENDERTARGETS:4,5 */
    }
#endif