#if defined SHADERS_VERTEX
    out vec2 coord;

    void main(){
        coord = layout_texcoord.xy;

        gl_Position = vec4(coord * 2.0 - 1.0, 0.0, 1.0);
    }
#elif defined SHADERS_FRAGMENT
    in vec2 coord;
    
    #include "/program/shaders2/common/aces/aces.glsl"
    #include "/program/shaders2/common/bloom.glsl"
    #include "/program/shaders2/common/color_correction.glsl"
    #include "/program/shaders2/common/tonemap.glsl"
    #include "/program/shaders2/common/dof.glsl"

    vec3 tonemap(vec3 color){
        #if TONEMAP_TYPE == 0
            color = aces_tonemap(color);
        #elif TONEMAP_TYPE == 1
            color = filmic(color);
        #elif TONEMAP_TYPE == 2
            color = lottes(color);
        #elif TONEMAP_TYPE == 3
            color = uchimura(color);
        #elif TONEMAP_TYPE == 4
            color = curve_tonemap(color);
        #endif
        
        return color;
    }

    vec3 rain_render(vec2 coord, vec3 color, vec3 bloom){
        #ifdef BLOOM
            return color + texture2(colortex15, coord).w * bloom * wetness;
        #else 
            return color + texture2(colortex15, coord).w * 0.2 * wetness;
        #endif
    }

    vec3 average_bloom(vec2 coord, vec3 color, vec3 bloom){
        bloomSettings b = s_bloom();
        
        float average_exposure = texture2(colortex5, coord).w;
        float average_bloom = 0.05 + b.strength;
        
        #ifdef UNDERWATER_BLOOM
            average_bloom = isEyeInWater == 1 ? isEyeInWater * (0.5 + b.underwater_strength) : average_bloom;
            average_bloom = isEyeInWater == 2 ? average_bloom : average_bloom;
        #endif

        return mix(color, bloom, average_bloom);
        //return texture2(colortex8, coord).xyz;
    }

    vec3 desaturation(vec2 coord, vec3 color){
        float average_exposure = texture2(colortex5, coord).w;

        vec3 desaturation = luminance(color) * vec3(0.5, 0.7, 1.0);
        vec3 desaturation_amt = saturate(exp(-0.75 * desaturation * average_exposure));
        
        #if defined OVER_WORLD
            #if EXPOSURE_TYPE == 0
                return mix(color, desaturation, desaturation_amt);
            #elif EXPOSURE_TYPE == 1
                return color;
            #endif
        #else
            return color;
        #endif
    }
    
    void main(){
        vec3 bloom = bloom(coord);
        vec3 color = depth_of_field(coord);

             color = average_bloom(coord, color, bloom);
             color = rain_render(coord, color, bloom);
             color = desaturation(coord, color);
             color = tonemap(color);
             color = post_process(color);

        buffers0.xyz = color;
        buffers0.w = 1.0;

        /* RENDERTARGETS:0 */
    }
#endif